from typing import Type

from sqlalchemy import MetaData
from cs_mcp.db.models.jinryo.jy import Jy


class TableManager:
    def __init__(self):
        self._jy_cache = {}

    def get_jy(self, schema: str, ym: str) -> Type[Jy]:
        cache_key = f"{schema}.jy{ym}"

        # 캐시에서 확인
        if cache_key in self._jy_cache:
            return self._jy_cache[cache_key]

        # 새로운 Jy 클래스 생성
        class JyDynamic(Jy):
            metadata = MetaData(schema=schema)
            __tablename__ = f"jy{ym}"  # type: ignore

        # 캐시에 저장
        self._jy_cache[cache_key] = JyDynamic

        return JyDynamic



from typing import Any, Callable
import pandas as pd
from sqlalchemy import desc, select
from sqlalchemy.ext.asyncio import AsyncSession
from sqlalchemy.orm import load_only
from datetime import datetime
from dateutil.relativedelta import relativedelta

from cs_mcp.constants import inpatient_group_dict, ldl_description_dict, ldl_item_dict, ldl_keys
from cs_mcp.db.dependencies import get_db
from cs_mcp.db.models import LtcDrgList
from cs_mcp.tools.types import InpatientGroupType, LdlItemType


async def get_ltcdrg_items(symd: str, eymd: str, field: Any, where: Callable[[], Any]):
  async with get_db() as session:
    query = select(LtcDrgList) \
        .where(LtcDrgList.ldl_startymd >= symd) \
        .where(LtcDrgList.ldl_startymd <= eymd) \
        .where(LtcDrgList.ldl_index == "0") \
        .where(where()) \
        .options(load_only(LtcDrgList.ldl_chart, LtcDrgList.ldl_startymd, field))
    result = await session.scalars(query)
    return result.all()


async def get_ltcdrg_by_id(session: AsyncSession, id: int):
    # session 에서 wj 객체를 가져온다.
  ldl = await session.scalar(
      select(LtcDrgList)
      .where((LtcDrgList.ldl_auto == id))
  )
  return ldl


async def get_adl_scores(charts: list[str], symd: str, eymd: str):
  async with get_db() as session:
    query = select(LtcDrgList).where(
        (LtcDrgList.ldl_startymd >= symd)
        & (LtcDrgList.ldl_startymd <= eymd)
        & (LtcDrgList.ldl_index == "0")
        & (LtcDrgList.ldl_adl > 0)
        & (LtcDrgList.ldl_dc != "1")
    )
    if charts:
      query = query.where(LtcDrgList.ldl_chart.in_(charts))
    query.options(load_only(LtcDrgList.ldl_chart,
                  LtcDrgList.ldl_startymd, LtcDrgList.ldl_adl))

    ldls = await session.scalars(query)
    ldl_list = [(ldl.ldl_chart, ldl.ldl_startymd, ldl.ldl_adl)
                for ldl in ldls if (not charts or ldl.ldl_chart in charts)]
    df = pd.DataFrame(ldl_list, columns=["차트번호", "시작일자", "ADL"])
    return df.to_markdown(index=False)


async def get_ltcdrg_dates(chart: str):
  async with get_db() as session:
    # session 에서 wj 객체를 가져온다.
    ldls = await session.scalars(
        select(LtcDrgList)
        .where(
            (LtcDrgList.ldl_chart == chart) & (
                LtcDrgList.ldl_dc != "1")
        )
        .order_by(desc(LtcDrgList.ldl_startymd))
        .options(
            load_only(
                LtcDrgList.ldl_auto, LtcDrgList.ldl_startymd, LtcDrgList.ldl_yuhyung
            )
        )
    )
    return [
        {"Id": ldl.ldl_auto, "일자": ldl.ldl_startymd, "유형": ldl.ldl_yuhyung}
        for ldl in ldls
    ]


async def compare_ltcdrg(id_a: int, id_b: int):
  async with get_db() as session:
    # session 에서 wj 객체를 가져온다.
    ldl_a = await get_ltcdrg_by_id(session, id_a)
    ldl_b = await get_ltcdrg_by_id(session, id_b)

    if ldl_a is None or ldl_b is None:
      return "비교할 장기요양점검표가 없습니다."

    # LDL 필드에 대한 설명을 담은 dictionary
    ldl_a_dict = ldl_a.to_dict()
    ldl_b_dict = ldl_b.to_dict()

    different_keys = []
    for key in ldl_keys:
      if ldl_a_dict.get(key) != ldl_b_dict.get(key):
        different_keys.append(key)

    data = {
        f"{ldl_a.ldl_startymd}({ldl_a.ldl_auto})": [
            ldl_a_dict[key] for key in different_keys
        ],
        f"{ldl_b.ldl_startymd}({ldl_b.ldl_auto})": [
            ldl_b_dict[key] for key in different_keys
        ],
        "description": [ldl_description_dict[key] for key in different_keys],
    }

    df = pd.DataFrame(data)

    return df.to_markdown(index=False)


async def search_ltcdrg_items(symd: str, eymd: str, items: list[LdlItemType]):
  item_dict: dict[str, dict[str, Any]] = {}

  field_names = set[str]()
  for item in items:
    field_name = ldl_item_dict[item]
    field_names.add(field_name)
    field = getattr(LtcDrgList, field_name)

    result = []
    if item == "장기요양등급" or item == "망상" or item == "혼수" or item == "단기기억력장애" or item == "인식기술" or item == "이해시키는능력" or item == "의사표현" or item == "섬망":
      result = await get_ltcdrg_items(symd, eymd, field, where=lambda: field > "0")

    for r in result:
      key = f"{r.ldl_chart}_{r.ldl_startymd}"
      if key in item_dict:
        item_dict[key] = {**item_dict[key],
                          field_name: getattr(r, field_name)}
      else:
        item_dict[key] = {field_name: getattr(r, field_name)}

    result = [(row.ldl_chart, row.ldl_startymd, getattr(row, field_name))
              for row in result]
    # return result

  result_arrs = []
  for key, item in item_dict.items():
    chart, startymd = key.split("_")

    value_arr = []
    for field_name in field_names:
      value_arr.append(item.get(field_name, "None"))

    result_arrs.append([chart, startymd, *value_arr])

  descriptions = [ldl_description_dict.get(
      field_name, "") for field_name in field_names]

  return pd.DataFrame(result_arrs, columns=["차트번호", "시작일자", *descriptions]).to_markdown(index=False)


async def get_ldl_with_adl(symd: str, eymd: str):
  async with get_db() as session:
    result = await session.scalars(
        select(LtcDrgList)
        .where(LtcDrgList.ldl_startymd >= symd)
        .where(LtcDrgList.ldl_startymd <= eymd)
        .where(LtcDrgList.ldl_adl > 0)
        .where(LtcDrgList.ldl_index == "0")
        .options(load_only(LtcDrgList.ldl_chart, LtcDrgList.ldl_startymd, LtcDrgList.ldl_adl))
    )
    return result.all()


async def compare_adl(symd_a: str, eymd_a: str, symd_b: str, eymd_b: str, status: str = "Increase"):
  # a월 데이터 조회
  list_a = await get_ldl_with_adl(symd_a, eymd_a)
  list_b = await get_ldl_with_adl(symd_b, eymd_b)

  charts = {ldl.ldl_chart for ldl in list_a} | {
      ldl.ldl_chart for ldl in list_b}

  records = []

  for chart in charts:
    a_scores = [
        (ldl.ldl_startymd, ldl.ldl_adl) for ldl in list_a if ldl.ldl_chart == chart and ldl.ldl_adl]
    b_scores = [
        (ldl.ldl_startymd, ldl.ldl_adl) for ldl in list_b if ldl.ldl_chart == chart and ldl.ldl_adl]

    if a_scores:
      max_a = max(a_scores, key=lambda x: x[1])
      a_startymd, max_a_score = max_a
    else:
      a_startymd, max_a_score = "", 0

    if b_scores:
      max_b = max(b_scores, key=lambda x: x[1])
      b_startymd, max_b_score = max_b
    else:
      b_startymd, max_b_score = "", 0

    if status == "Increase":
      if a_scores and b_scores and max_b_score > max_a_score:
        records.append(
            (chart, a_startymd, max_a_score, b_startymd, max_b_score))
    elif status == "Decrease":
      if a_scores and b_scores and max_b_score < max_a_score:
        records.append(
            (chart, a_startymd, max_a_score, b_startymd, max_b_score))
    else:  # Unchanged
      if a_scores and b_scores and max_b_score == max_a_score:
        records.append(
            (chart, a_startymd, max_a_score, b_startymd, max_b_score))

  df = pd.DataFrame(
      records, columns=["차트번호", "A 시작일자", "A ADL", "B 시작일자", "B ADL"])
  return df.to_markdown(index=False)


async def get_ldl_with_inpatient_group(symd: str, eymd: str, grade_code: str = ""):
  async with get_db() as session:
    query = select(LtcDrgList)
    query = query.where(LtcDrgList.ldl_startymd >= symd)
    query = query.where(LtcDrgList.ldl_startymd <= eymd)
    query = query.where(LtcDrgList.ldl_etc4 != "")
    query = query.where(LtcDrgList.ldl_index == "0")
    if grade_code:
      query = query.where(LtcDrgList.ldl_etc4.like(f"{grade_code}%"))
    query = query.options(load_only(LtcDrgList.ldl_chart,
                          LtcDrgList.ldl_startymd, LtcDrgList.ldl_etc4))
    result = await session.scalars(query)
    return result.all()


async def compare_inpatient_group(symd_a: str, eymd_a: str, group_a: str, symd_b: str, eymd_b: str, group_b: str, status: str = "Increase"):
  # 등급 점수 (높은 등급일수록 높은 점수)
  grade_score = {"A1": 5, "A2": 4, "A3": 3, "A6": 2, "A7": 1}

  a_code = inpatient_group_dict.get(group_a, "")
  b_code = inpatient_group_dict.get(group_b, "")

  # a월 데이터 조회 (group_a에 해당하는 등급만)
  list_a = await get_ldl_with_inpatient_group(symd_a, eymd_a, a_code)

  list_b = await get_ldl_with_inpatient_group(symd_b, eymd_b, b_code)

  charts = {ldl.ldl_chart for ldl in list_a} | {
      ldl.ldl_chart for ldl in list_b}

  records = []

  for chart in charts:
    a_grades = [
        (ldl.ldl_startymd, ldl.ldl_etc4) for ldl in list_a if ldl.ldl_chart == chart and ldl.ldl_etc4]
    b_grades = [
        (ldl.ldl_startymd, ldl.ldl_etc4) for ldl in list_b if ldl.ldl_chart == chart and ldl.ldl_etc4]

    if a_grades:
      # 가장 최근 등급 (startymd가 큰 것)
      latest_a = max(a_grades, key=lambda x: x[0])
      a_startymd, a_grade = latest_a
      a_level = grade_score.get(a_grade[:2], 0)  # A1, A2 등
    else:
      a_startymd, a_grade, a_level = "", "", 0

    if b_grades:
      latest_b = max(b_grades, key=lambda x: x[0])
      b_startymd, b_grade = latest_b
      b_level = grade_score.get(b_grade[:2], 0)
    else:
      b_startymd, b_grade, b_level = "", "", 0

    if status == "Increase":
      # 등급이 높아짐 (점수가 높아짐)
      if a_grades and b_grades and b_level > a_level:
        records.append((chart, a_startymd, a_grade, b_startymd, b_grade))
    elif status == "Decrease":
      # 등급이 낮아짐 (점수가 낮아짐)
      if a_grades and b_grades and b_level < a_level:
        records.append((chart, a_startymd, a_grade, b_startymd, b_grade))
    else:  # Unchanged
      if a_grades and b_grades and b_level == a_level:
        records.append((chart, a_startymd, a_grade, b_startymd, b_grade))

  df = pd.DataFrame(
      records, columns=["차트번호", "A 시작일자", "A 등급", "B 시작일자", "B 등급"])
  return df.to_markdown(index=False)


async def get_inpatient_groups(symd: str, eymd: str, group: InpatientGroupType):
  code = inpatient_group_dict.get(group, "")
  async with get_db() as session:
    query = (
        select(LtcDrgList)
        .where(LtcDrgList.ldl_startymd >= symd)
        .where(LtcDrgList.ldl_startymd <= eymd)
        .where(LtcDrgList.ldl_etc4 != "")
        .where(LtcDrgList.ldl_index == "0")
        .where(LtcDrgList.ldl_etc4.like(f"{code}%"))
        .options(load_only(LtcDrgList.ldl_chart, LtcDrgList.ldl_startymd, LtcDrgList.ldl_etc4))
    )
    result = await session.scalars(query)
    ldl_list = [
        (ldl.ldl_chart, ldl.ldl_startymd, ldl.ldl_etc4)
        for ldl in result
    ]
    total_count = len(ldl_list)
    unique_patients = len(set(chart for chart, _, _ in ldl_list))
    df = pd.DataFrame(ldl_list, columns=["차트번호", "시작일자", "분류코드"])
    summary = f"총 수량: {total_count}\n환자수: {unique_patients}\n\n"
    return summary + df.to_markdown(index=False)


async def check_dementia_due_date(
    dementia_check_type: str,
    reference: str,
    check_months: int,
    prior_to: int,
):
    """장기요양점검표 치매검사 만료일자를 확인합니다."""

    field_dict = {
        "MMSE": {
            "yn": "ldl_c6_a",
            "date": "ldl_c6_b2"
        },
        "CDR": {
            "yn": "ldl_c7_a",
            "date": "ldl_c7_b2"
        },
        "GDS": {
            "yn": "ldl_c7_c",
            "date": "ldl_c7_d2"
        },
    }
    fields = field_dict[dementia_check_type]
    yn_field = fields['yn']
    date_field = fields['date']

    yn_attr = getattr(LtcDrgList, yn_field)
    date_attr = getattr(LtcDrgList, date_field)

    # 6개월 전 날짜 계산
    reference_date = datetime.strptime(reference, "%Y%m%d").date()
    reference_date = reference_date - relativedelta(months=check_months)
    symd = reference_date.strftime("%Y%m%d")

    async with get_db() as session:
      query = select(LtcDrgList).where(
          (yn_attr == "1") &
          (date_attr != None) &
          (date_attr != "") &
          (LtcDrgList.ldl_dc != "1") &
          (LtcDrgList.ldl_startymd > symd) &
          (LtcDrgList.ldl_startymd <= reference)
      ).options(load_only(LtcDrgList.ldl_chart, LtcDrgList.ldl_startymd, date_attr))
      results = await session.scalars(query)

      custom_datas = []
      for r in results:
        date = getattr(r, date_field)
        due_date = datetime.strptime(
            date, "%Y%m%d").date() + relativedelta(months=check_months)

        # due_date - rdate 일수가 14일 보다 작거나 같은 경우
        if (due_date - reference_date).days <= prior_to:
          custom_datas.append({
              "차트번호": r.ldl_chart,
              "작성일자": r.ldl_startymd,
              f"{dementia_check_type} 검사일자": date,
              "만료일자": due_date.strftime("%Y%m%d"),
              "남은일수": (due_date - reference_date).days
          })

      # custom_datas 를 pd table 로 변환
      df = pd.DataFrame(custom_datas)
      if df.empty:
        return "조회된 데이터가 없습니다."
      df = df.sort_values(by=["남은일수", "차트번호"])
      return df.to_markdown(index=False)



from typing import Annotated, Any
from fastmcp import FastMCP
from pydantic import Field
from sqlalchemy import and_, select

from cs_mcp.db.dependencies import get_db
from cs_mcp.db.models.table_manager import TableManager
from cs_mcp.tools.types import ChartNumberType



def jy_tools(mcp: FastMCP[Any]):
  @mcp.tool()
  async def get_jy(
      chart_number: ChartNumberType,
      saup: Annotated[str, Field(description="사업자", max_length=2)],
      isHanbang: Annotated[bool, Field(description="한방: True")],
      year_month: Annotated[str, Field(description="년월(YYYYMM)", max_length=6)],
      start_day: Annotated[
          str, Field(description="당월 기간 중 처음일(DD)", max_length=2)
      ],
      end_day: Annotated[
          str, Field(description="당월 기간 중 마지막일(DD)", max_length=2)
      ],
  ):
    """차트번호로 진료내역(처방내역)을 조회합니다."""
    async with get_db() as session:
      dbcode = "han" if isHanbang else "jinryo"
      dbname = f"{dbcode}{saup}"
      start_ymd = f"{year_month}{start_day}"
      end_ymd = f"{year_month}{end_day}"
      Jy = TableManager().get_jy(dbname, year_month)

      jy = await session.execute(
          select(Jy).where(
              (Jy.jy_chart == chart_number)
              & (Jy.jy_ymd >= start_ymd)
              & (Jy.jy_ymd <= end_ymd)
              & (Jy.jy_dc != "1")
          )
      )

      jys = jy.scalars().all()
    if not jys:
      return f"차트번호 {chart_number}에 해당하는 진료내역이 없습니다."

    return [
        {
            "id": jy.jy_auto,
            "진료일자": jy.jy_ymd,  # type: ignore
            "처방코드": jy.jy_yscode,
            "진료내용": jy.jy_myung,  # type: ignore
        }
        for jy in jys
    ]

  @mcp.tool()
  async def del_jy(
      chart_number: ChartNumberType,
      saup: Annotated[str, Field(description="사업자", max_length=2)],
      isHanbang: Annotated[bool, Field(description="한방: True")],
      year_month: Annotated[str, Field(description="년월(YYYYMM)", max_length=6)],
      ids: Annotated[list[int], Field(description="삭제할 진료내역 ID 목록")],
  ):
    """차트번호로 진료내역(처방내역)을 삭제합니다."""
    async with get_db() as session:
      dbcode = "han" if isHanbang else "jinryo"
      dbname = f"{dbcode}{saup}"
      Jy = TableManager().get_jy(dbname, year_month)

      jys = await session.scalars(
          select(Jy).where(
              and_(
                  Jy.jy_chart == chart_number,
                  Jy.jy_dc != "1",
                  Jy.jy_auto.in_(ids),  # type: ignore
              )
          )
      )

      if not jys:
        return f"차트번호 {chart_number}에 해당하는 진료내역이 없습니다."

      for jy in jys:
        jy.jy_dc = "1"
        session.add(jy)

      await session.commit()

    return f"차트번호 {chart_number}의 진료내역을 삭제했습니다."



import httpx

from typing import Annotated, Any
from fastmcp import FastMCP
from pydantic import Field


def advice_tools(mcp: FastMCP[Any]):
  @mcp.tool()
  async def advice_eclick(question: Annotated[str, Field(description="질문")], page: Annotated[int, Field(description="페이지", ge=1, le=3)] = 1):
    """
    eclick 질문에 대한 응답을 가져옵니다.

    만일 질문에 대한 정확한 응답이 없으면 페이지를 늘려서 조회하세요.
    """
    url = "https://ai.click-soft.co.kr/vector-store/api/advices"
    async with httpx.AsyncClient() as client:
      response = await client.post(url, json={"question": question, "page": page})
      response.raise_for_status()
    return response.json()



from cs_mcp.db.models.base import Base
from typing import Optional
from cs_mcp.db.sqlalchemy import mc, M
from sqlalchemy import String


class LtcDrgList(Base):
  __tablename__ = "ltcdrglist"  # type: ignore

  ldl_auto: M[Optional[int]] = mc(
      default=None, primary_key=True, autoincrement=True
  )
  ldl_version: M[str] = mc(String(6), default="")
  ldl_ymd: M[str] = mc(String(8))
  ldl_saup: M[str] = mc(String(2))
  ldl_chart: M[str] = mc(String(8))
  ldl_yuhyung: M[str] = mc(String(2))
  ldl_ibymd: M[str] = mc(String(8))
  ldl_startymd: M[str] = mc(String(8))
  ldl_jinchal: M[str] = mc(String(2), default="")
  ldl_index: M[str] = mc(String(1), default="0")
  ldl_change: M[str] = mc(String(1), default="0")
  ldl_beginymd: M[str] = mc(String(8))
  ldl_yoyangkiho: M[Optional[str]] = mc(String(8), default="")
  ldl_suname: M[Optional[str]] = mc(String(50), default="")
  ldl_sex: M[Optional[str]] = mc(String(1), default="")
  ldl_nai: M[Optional[str]] = mc(String(3), default="")
  ldl_jumin: M[Optional[str]] = mc(String(48), default="")
  ldl_byungsil: M[Optional[str]] = mc(String(15), default="")
  ldl_gubun: M[Optional[str]] = mc(String(1), default="")
  ldl_stay: M[Optional[str]] = mc(String(1), default="")
  ldl_career: M[Optional[str]] = mc(String(1), default="")
  ldl_bphigh: M[Optional[int]] = mc(default=0)
  ldl_bplow: M[Optional[int]] = mc(default=0)
  ldl_a10_a: M[Optional[int]] = mc(default=0)
  ldl_a10_b: M[Optional[int]] = mc(default=0)
  ldl_a10_c: M[Optional[int]] = mc(default=0)
  ldl_a10_d: M[Optional[int]] = mc(default=0)
  ldl_a11: M[Optional[int]] = mc(default=1)
  ldl_a12_a: M[Optional[int]] = mc(default=0)
  ldl_a12_b1: M[Optional[int]] = mc(default=0)
  ldl_a12_b2: M[Optional[int]] = mc(default=0)
  ldl_a12_b3: M[Optional[int]] = mc(default=0)
  ldl_a12_b4: M[Optional[int]] = mc(default=0)
  ldl_a12_b5: M[Optional[int]] = mc(default=0)
  ldl_a12_b6: M[Optional[int]] = mc(default=0)
  ldl_a12_b7: M[Optional[int]] = mc(default=0)
  ldl_a12_b8: M[Optional[int]] = mc(default=0)
  ldl_a13: M[Optional[int]] = mc(default=0)
  ldl_a14_a: M[Optional[int]] = mc(default=0)
  ldl_a14_b: M[Optional[int]] = mc(default=0)
  ldl_a14_c: M[Optional[int]] = mc(default=0)
  ldl_a14_d: M[Optional[int]] = mc(default=0)
  ldl_a14_e: M[Optional[int]] = mc(default=0)
  ldl_a14_f: M[Optional[int]] = mc(default=0)
  ldl_a14_g: M[Optional[int]] = mc(default=0)
  ldl_b1: M[Optional[int]] = mc(default=0)
  ldl_b2: M[Optional[str]] = mc(String(1), default="")
  ldl_c1: M[Optional[str]] = mc(String(1), default="")
  ldl_c2: M[Optional[str]] = mc(String(1), default="")
  ldl_c3: M[Optional[str]] = mc(String(1), default="")
  ldl_c4: M[Optional[str]] = mc(String(1), default="")
  ldl_c5_a: M[Optional[str]] = mc(String(1), default="")
  ldl_c5_b: M[Optional[str]] = mc(String(1), default="")
  ldl_c5_c: M[Optional[str]] = mc(String(1), default="")
  ldl_c5_d: M[Optional[str]] = mc(String(1), default="")
  ldl_c5_e: M[Optional[str]] = mc(String(1), default="")
  ldl_c5_f: M[Optional[str]] = mc(String(1), default="")
  ldl_c5_g: M[Optional[str]] = mc(String(1), default="")
  ldl_c5_h: M[Optional[str]] = mc(String(1), default="")
  ldl_c5_i: M[Optional[str]] = mc(String(1), default="")
  ldl_c5_j: M[Optional[str]] = mc(String(1), default="")
  ldl_c5_k: M[Optional[str]] = mc(String(1), default="")
  ldl_c5_l: M[Optional[str]] = mc(String(1), default="")
  ldl_c5_m: M[Optional[str]] = mc(String(1), default="")
  ldl_c5_n: M[Optional[str]] = mc(String(1), default="")
  ldl_c6_a: M[Optional[str]] = mc(String(1), default="")
  ldl_c6_b2: M[Optional[str]] = mc(String(8), default="")
  ldl_c7_a: M[Optional[int]] = mc(default=0)
  ldl_c7_b1: M[Optional[float]] = mc(default=0)
  ldl_c7_b2: M[Optional[str]] = mc(String(8), default="")
  ldl_c7_c: M[Optional[int]] = mc(default=0)
  ldl_c7_d1: M[Optional[int]] = mc(default=0)
  ldl_c7_d2: M[Optional[str]] = mc(String(8), default="")
  ldl_d1: M[Optional[int]] = mc(default=0)
  ldl_d2: M[Optional[int]] = mc(default=0)
  ldl_d3: M[Optional[int]] = mc(default=0)
  ldl_d4: M[Optional[int]] = mc(default=0)
  ldl_d5: M[Optional[int]] = mc(default=0)
  ldl_d6: M[Optional[int]] = mc(default=0)
  ldl_d7: M[Optional[int]] = mc(default=0)
  ldl_d8: M[Optional[int]] = mc(default=0)
  ldl_d9: M[Optional[int]] = mc(default=0)
  ldl_d10: M[Optional[int]] = mc(default=0)
  ldl_d11: M[Optional[int]] = mc(default=0)
  ldl_d12_a: M[Optional[int]] = mc(default=0)
  ldl_d12_b: M[Optional[int]] = mc(default=0)
  ldl_d12_c: M[Optional[int]] = mc(default=0)
  ldl_e1: M[Optional[int]] = mc(default=0)
  ldl_e2: M[Optional[int]] = mc(default=0)
  ldl_e3_a: M[Optional[int]] = mc(default=0)
  ldl_e3_b: M[Optional[int]] = mc(default=0)
  ldl_e3_c: M[Optional[int]] = mc(default=0)
  ldl_e3_d: M[Optional[int]] = mc(default=0)
  ldl_e3_e: M[Optional[int]] = mc(default=0)
  ldl_e3_f: M[Optional[int]] = mc(default=0)
  ldl_e3_g: M[Optional[int]] = mc(default=0)
  ldl_e3_g1: M[Optional[str]] = mc(String(8), default="")
  ldl_e3_g1_1: M[str] = mc(String(8))
  ldl_e3_g1_2: M[str] = mc(String(8))
  ldl_e3_g2_1: M[str] = mc(String(8))
  ldl_e3_g2_2: M[str] = mc(String(8))
  ldl_e3_g3_1: M[str] = mc(String(8))
  ldl_e3_g3_2: M[str] = mc(String(8))
  ldl_e3_g4_1: M[str] = mc(String(8))
  ldl_e3_g4_2: M[str] = mc(String(8))
  ldl_e3_g5_1: M[str] = mc(String(8))
  ldl_e3_g5_2: M[str] = mc(String(8))
  ldl_e3_g6_1: M[str] = mc(String(8))
  ldl_e3_g6_2: M[str] = mc(String(8))
  ldl_e3_g7_1: M[str] = mc(String(8))
  ldl_e3_g7_2: M[str] = mc(String(8))
  ldl_e3_g8_1: M[str] = mc(String(8))
  ldl_e3_g8_2: M[str] = mc(String(8))
  ldl_e3_g9_1: M[str] = mc(String(8))
  ldl_e3_g9_2: M[str] = mc(String(8))
  ldl_e3_g10_1: M[str] = mc(String(8))
  ldl_e3_g10_2: M[str] = mc(String(8))
  ldl_f1_a1_a: M[Optional[str]] = mc(String(1), default="")
  ldl_f1_a2_a: M[Optional[str]] = mc(String(1), default="")
  ldl_f1_a2_b2: M[Optional[str]] = mc(String(8), default="")
  ldl_f1_b: M[Optional[int]] = mc(default=0)
  ldl_f1_c: M[Optional[int]] = mc(default=0)
  ldl_f1_d: M[Optional[int]] = mc(default=0)
  ldl_f1_e: M[Optional[int]] = mc(default=0)
  ldl_f1_f: M[Optional[int]] = mc(default=0)
  ldl_f1_g: M[Optional[int]] = mc(default=0)
  ldl_f1_h: M[Optional[int]] = mc(default=0)
  ldl_f1_i: M[Optional[int]] = mc(default=0)
  ldl_f1_j: M[Optional[int]] = mc(default=0)
  ldl_f1_k: M[Optional[int]] = mc(default=0)
  ldl_f1_l: M[Optional[int]] = mc(default=0)
  ldl_f1_m: M[Optional[int]] = mc(default=0)
  ldl_f1_n: M[Optional[int]] = mc(default=0)
  ldl_f1_o: M[Optional[int]] = mc(default=0)
  ldl_f1_p: M[Optional[int]] = mc(default=0)
  ldl_f1_q: M[Optional[int]] = mc(default=0)
  ldl_f1_r: M[Optional[int]] = mc(default=0)
  ldl_f1_s: M[Optional[int]] = mc(default=0)
  ldl_f1_t: M[Optional[int]] = mc(default=0)
  ldl_f1_u: M[Optional[int]] = mc(default=0)
  ldl_f1_v: M[Optional[int]] = mc(default=0)
  ldl_f1_w: M[Optional[int]] = mc(default=0)
  ldl_f1_x: M[Optional[int]] = mc(default=0)
  ldl_f1_y: M[Optional[int]] = mc(default=0)
  ldl_f1_z: M[Optional[int]] = mc(default=0)
  ldl_f1_aa: M[Optional[int]] = mc(default=0)
  ldl_f1_ab: M[Optional[int]] = mc(default=0)
  ldl_f2_a: M[Optional[int]] = mc(default=0)
  ldl_f2_b: M[Optional[int]] = mc(default=0)
  ldl_f2_c: M[Optional[int]] = mc(default=0)
  ldl_f2_d: M[Optional[int]] = mc(default=0)
  ldl_f2_e: M[Optional[int]] = mc(default=0)
  ldl_f2_f: M[Optional[int]] = mc(default=0)
  ldl_f2_g: M[Optional[int]] = mc(default=0)
  ldl_f2_h: M[Optional[int]] = mc(default=0)
  ldl_g1_a: M[Optional[int]] = mc(default=0)
  ldl_g1_a1: M[Optional[float]] = mc(default=0)
  ldl_g1_a2: M[Optional[int]] = mc(default=0)
  ldl_g1_a3: M[int] = mc()
  ldl_g1_b: M[Optional[int]] = mc(default=0)
  ldl_g1_c: M[Optional[int]] = mc(default=0)
  ldl_g1_d: M[Optional[int]] = mc(default=0)
  ldl_g1_e: M[Optional[int]] = mc(default=0)
  ldl_g1_f: M[Optional[int]] = mc(default=0)
  ldl_g1_g: M[Optional[int]] = mc(default=0)
  ldl_g2_a: M[Optional[int]] = mc(default=0)
  ldl_g2_b: M[Optional[int]] = mc(default=0)
  ldl_g2_b1: M[Optional[int]] = mc(default=0)
  ldl_g2_b2: M[Optional[int]] = mc(default=0)
  ldl_g2_b3: M[Optional[int]] = mc(default=0)
  ldl_g2_c: M[Optional[int]] = mc(default=0)
  ldl_g2_c1: M[int] = mc()
  ldl_g2_c2: M[int] = mc()
  ldl_g3_a: M[Optional[int]] = mc(default=0)
  ldl_g3_b: M[Optional[int]] = mc(default=0)
  ldl_g4: M[Optional[int]] = mc(default=0)
  ldl_h1: M[Optional[int]] = mc(default=0)
  ldl_h21_a: M[Optional[int]] = mc(default=0)
  ldl_h21_b1: M[Optional[float]] = mc(default=0)
  ldl_h21_b2: M[Optional[str]] = mc(String(8), default="")
  ldl_h22: M[Optional[int]] = mc(default=0)
  ldl_h23_a: M[Optional[int]] = mc(default=0)
  ldl_h23_b1: M[Optional[float]] = mc(default=0)
  ldl_h23_b2: M[Optional[str]] = mc(String(8), default="")
  ldl_h3_a: M[Optional[int]] = mc(default=0)
  ldl_h3_b: M[Optional[int]] = mc(default=0)
  ldl_h3_b1: M[int] = mc()
  ldl_h3_b2: M[int] = mc()
  ldl_h4_a: M[Optional[str]] = mc(String(1), default="")
  ldl_h4_b: M[Optional[str]] = mc(String(1), default="")
  ldl_i1_a1: M[Optional[int]] = mc(default=0)
  ldl_i1_a2: M[Optional[int]] = mc(default=0)
  ldl_i1_a3: M[Optional[int]] = mc(default=0)
  ldl_i1_a4: M[Optional[int]] = mc(default=0)
  ldl_i1_b1: M[Optional[int]] = mc(default=0)
  ldl_i1_b2: M[Optional[int]] = mc(default=0)
  ldl_i1_b3: M[Optional[int]] = mc(default=0)
  ldl_i1_b4: M[Optional[int]] = mc(default=0)
  ldl_i2_a: M[Optional[int]] = mc(default=0)
  ldl_i2_b: M[Optional[str]] = mc(String(8), default="")
  ldl_i3: M[Optional[int]] = mc(default=0)
  ldl_i4_a: M[Optional[int]] = mc(default=0)
  ldl_i4_b: M[Optional[int]] = mc(default=0)
  ldl_i4_b1: M[Optional[int]] = mc(default=0)
  ldl_i4_b2: M[Optional[int]] = mc(default=0)
  ldl_i4_c: M[Optional[int]] = mc(default=0)
  ldl_i4_d: M[Optional[int]] = mc(default=0)
  ldl_i4_e: M[Optional[int]] = mc(default=0)
  ldl_i5_a: M[Optional[int]] = mc(default=0)
  ldl_i5_b: M[Optional[int]] = mc(default=0)
  ldl_i5_c: M[Optional[int]] = mc(default=0)
  ldl_i5_d: M[Optional[int]] = mc(default=0)
  ldl_i5_d1: M[Optional[int]] = mc(default=0)
  ldl_i5_d2: M[Optional[int]] = mc(default=0)
  ldl_i5_e: M[Optional[int]] = mc(default=0)
  ldl_i5_e1: M[Optional[int]] = mc(default=0)
  ldl_i5_e2: M[Optional[int]] = mc(default=0)
  ldl_i5_f: M[Optional[int]] = mc(default=0)
  ldl_i5_g: M[Optional[int]] = mc(default=0)
  ldl_i5_h: M[int] = mc()
  ldl_j1: M[Optional[int]] = mc(default=0)
  ldl_j2: M[Optional[int]] = mc(default=0)
  ldl_j3: M[Optional[int]] = mc(default=0)
  ldl_j4: M[Optional[int]] = mc(default=0)
  ldl_k1_a: M[Optional[int]] = mc(default=0)
  ldl_k1_a1: M[int] = mc()
  ldl_k1_b: M[Optional[int]] = mc(default=0)
  ldl_k1_c: M[Optional[int]] = mc(default=0)
  ldl_k1_d: M[Optional[int]] = mc(default=0)
  ldl_k1_e: M[Optional[int]] = mc(default=0)
  ldl_k1_e1: M[Optional[float]] = mc(default=0)
  ldl_k1_e2: M[Optional[int]] = mc(default=0)
  ldl_k1_f: M[Optional[int]] = mc(default=0)
  ldl_k1_g: M[Optional[int]] = mc(default=0)
  ldl_k1_h: M[Optional[int]] = mc(default=0)
  ldl_k1_i: M[Optional[int]] = mc(default=0)
  ldl_k1_i1: M[int] = mc()
  ldl_k1_i2: M[int] = mc()
  ldl_k1_j: M[Optional[int]] = mc(default=0)
  ldl_k1_j1: M[Optional[int]] = mc(default=0)
  ldl_k1_j2: M[Optional[int]] = mc(default=0)
  ldl_k1_k: M[Optional[int]] = mc(default=0)
  ldl_k1_l: M[Optional[int]] = mc(default=0)
  ldl_k2: M[Optional[int]] = mc(default=0)
  ldl_adl: M[Optional[int]] = mc(default=0)
  ldl_doctor: M[Optional[int]] = mc(default=0)
  ldl_doctorname: M[Optional[str]] = mc(String(50), default="")
  ldl_tester: M[Optional[str]] = mc(String(20), default="")
  ldl_username: M[Optional[str]] = mc(String(20), default="")
  ldl_insertdate: M[Optional[str]] = mc(String(20), default="")
  ldl_updateuser: M[Optional[str]] = mc(String(20), default="")
  ldl_updatedate: M[Optional[str]] = mc(String(20), default="")
  ldl_docdate: M[Optional[str]] = mc(String(20), default="")
  ldl_nurdate: M[Optional[str]] = mc(String(20), default="")
  ldl_etc1: M[Optional[str]] = mc(String(255), default="")
  ldl_etc2: M[Optional[str]] = mc(String(255), default="")
  ldl_etc3: M[Optional[str]] = mc(String(255), default="")
  ldl_etc4: M[Optional[str]] = mc(String(255), default="")
  ldl_etc5: M[Optional[str]] = mc(String(255), default="")
  ldl_etc6: M[Optional[str]] = mc(String(255), default="")
  ldl_etc7: M[Optional[str]] = mc(String(255), default="")
  ldl_etc8: M[Optional[str]] = mc(default=None)
  ldl_etc9: M[Optional[str]] = mc(String(255), default="")
  ldl_etc10: M[Optional[str]] = mc(String(255), default="")
  ldl_dc: M[str] = mc(String(1), default="0")
  ldl_dcdate: M[Optional[str]] = mc(String(20), default="")
  ldl_dcuser: M[Optional[str]] = mc(String(20), default="")
  ldl_docuser: M[Optional[str]] = mc(String(20), default="")
  ldl_nuruser: M[Optional[str]] = mc(String(20), default="")
  ldl_insign: M[Optional[bytes]] = mc(default=None)
  ldl_upsign: M[Optional[bytes]] = mc(default=None)
  ldl_dcsign: M[Optional[bytes]] = mc(default=None)
  ldl_docsign: M[Optional[bytes]] = mc(default=None)
  ldl_nursign: M[Optional[bytes]] = mc(default=None)
  ldl_incert: M[Optional[str]] = mc(default=None)
  ldl_upcert: M[Optional[str]] = mc(default=None)
  ldl_dccert: M[Optional[str]] = mc(default=None)
  ldl_doccert: M[Optional[str]] = mc(default=None)
  ldl_nurcert: M[Optional[str]] = mc(default=None)
  ldl_etc11: M[Optional[str]] = mc(String(255), default="")
  ldl_etc12: M[Optional[str]] = mc(String(255), default="")
  ldl_etc13: M[Optional[str]] = mc(String(255), default="")
  ldl_etc14: M[Optional[str]] = mc(String(255), default="")
  ldl_etc15: M[Optional[str]] = mc(String(255), default="")
  ldl_enctype: M[Optional[str]] = mc(String(1), default="Y")
  ldl_linetext: M[Optional[str]] = mc(default=None)
  ldl_etc16: M[Optional[str]] = mc(String(255), default="")
  ldl_etc17: M[Optional[str]] = mc(String(255), default="")
  ldl_etc18: M[Optional[str]] = mc(String(255), default="")
  ldl_etc19: M[Optional[str]] = mc(String(255), default="")
  ldl_etc20: M[Optional[str]] = mc(String(255), default="")



from datetime import date, datetime
from typing import Annotated, Any, Literal
from dateutil.relativedelta import relativedelta
from fastmcp import FastMCP
from pydantic import Field
from sqlalchemy import select
from sqlalchemy.orm import load_only
from cs_mcp.db.dependencies import get_db
from cs_mcp.db.models import LtcDrgList
from cs_mcp.db.services import ltcdrg_service
from cs_mcp.tools.types import ChartNumberType, EymdType, InpatientGroupType, LdlItemType, SymdType


def ltcdrglist_tools(mcp: FastMCP[Any]):
  @mcp.tool()
  async def search_ltcdrg_items(
          symd: SymdType,
          eymd: EymdType,
          items: list[LdlItemType]):
    """
    장기요양 점검표에서 지정한 기간(symd~eymd) 내에 특정 항목(item)이 존재하는 경우를 조회합니다.
    """

    return await ltcdrg_service.search_ltcdrg_items(symd, eymd, items)

  @mcp.tool()
  async def get_adl_scores(
      charts: Annotated[list[ChartNumberType], Field(description="차트번호 목록")],
      symd: SymdType,
      eymd: EymdType,
  ):
    """장기요양 ADL 점수 조회

    args:
        charts:
          - 차트번호 목록
          - 빈 목록([])일 경우, 지정된 기간의 모든 환자 조회
    """

    return await ltcdrg_service.get_adl_scores(charts, symd, eymd)

  @mcp.tool()
  async def compare_ltcdrg(
      id_a: Annotated[int, Field(description="비교할 ID")],
      id_b: Annotated[int, Field(description="비교할 ID")],
  ):
    """장기요양점검표를 비교합니다.

    Args:
        id_a: First record ID.
        id_b: Second record ID.

    Output:
        Markdown table showing only the fields with different values, including field descriptions.
    """

    return await ltcdrg_service.compare_ltcdrg(id_a, id_b)

  @mcp.tool()
  async def get_ltcdrg_dates(
      chart_number: ChartNumberType,
  ):
    """특정 환자의 장기요양정검표 작성날짜를 조회합니다.

    args:
        chart_number: 차트번호(8자리 숫자)
    returns:
        Id: ID
        일자: 작성일자
        유형: 유형
    """
    return await ltcdrg_service.get_ltcdrg_dates(chart_number)

  @mcp.tool()
  async def compare_adl(
      symd_a: Annotated[str, Field(description="StartDate A(YYYYMMMM)", max_length=8)],
      eymd_a: Annotated[str, Field(description="EndDate A(YYYYMMMM)", max_length=8)],
      symd_b: Annotated[str, Field(description="StartDate B(YYYYMMMM)", max_length=8)],
      eymd_b: Annotated[str, Field(description="EndDate B(YYYYMMMM)", max_length=8)],
      status: Literal["Increase", "Decrease", "Unchanged"] = "Increase",
  ):
    """장기요양점검표 ADL 점수를 비교합니다.

    args:
      status: Status of comparison, "Increase" (default), "Decrease", or "Unchanged".
        - "Increase": Show records where ADL score increased from period A to B.
        - "Decrease": Show records where ADL score decreased from period A to B.
        - "Unchanged": Show records where ADL score remained the same from period A to B.
    """

    return await ltcdrg_service.compare_adl(symd_a, eymd_a, symd_b, eymd_b, status)

  @mcp.tool()
  async def get_inpatient_groups(
      symd: Annotated[str, Field(description="StartDate(YYYYMMMM)", max_length=8)],
      eymd: Annotated[str, Field(description="EndDate(YYYYMMMM)", max_length=8)],
      group: InpatientGroupType,
  ):
    """특정 기간 내 장기요양 입원군(군 분류)을 조회합니다.

    args:
        group: 장기요양 의료 등급 선택
            - None (default): 모든 등급 조회
            - "최고도": 최고도 등급
            - "고도": 고도 등급
            - "중도": 중도 등급
            - "경도": 경도 등급
            - "선택입원": 선택입원 등급
    returns:
        Markdown table with 차트번호, 시작일자, 장기요양등급, 군 분류
    """

    return await ltcdrg_service.get_inpatient_groups(symd, eymd, group)

  @mcp.tool()
  async def compare_inpatient_group(
      symd_a: Annotated[str, Field(description="StartDate A(YYYYMMMM)", max_length=8)],
      eymd_a: Annotated[str, Field(description="EndDate A(YYYYMMMM)", max_length=8)],
      group_a: InpatientGroupType,
      symd_b: Annotated[str, Field(description="StartDate B(YYYYMMMM)", max_length=8)],
      eymd_b: Annotated[str, Field(description="EndDate B(YYYYMMMM)", max_length=8)],
      group_b: InpatientGroupType,
      status: Literal["Increase", "Decrease", "Unchanged"] = "Increase",
  ):
    """장기요양 입원군(군 분류)을 비교합니다.

    args:
      status: Status of comparison, "Increase" (default), "Decrease", or "Unchanged".
        - "Increase": 등급이 높아진 경우 (예: 고도 → 최고도)
        - "Decrease": 등급이 낮아진 경우 (예: 최고도 → 고도)
        - "Unchanged": 등급이 변경되지 않은 경우
    """

    return await ltcdrg_service.compare_inpatient_group(symd_a, eymd_a, group_a, symd_b, eymd_b, group_b, status)

  @mcp.tool()
  async def check_dementia_due_date(
      dementia_check_type: Literal["MMSE", "GDR", "CDS"] = "MMSE",
      reference: Annotated[str, Field(
          description="YYYYMMDD", max_length=8)] = datetime.now().strftime("%Y%m%d"),
      check_months: Annotated[int, Field(
          description="Check Terms(Default: 6 Months)", ge=1, le=12)] = 6,
      prior_to: Annotated[int, Field(
          description="마감 x일 전(Default: 14 days)", ge=1, le=30)] = 14,
  ):
    """장기요양점검표 치매검사 만료일자를 확인합니다.

    args:
      reference: 기준일자(Default 오늘 날짜)
      previousMonth: 이전 월 기간(Default 6개월)
      dueDays: 도래일수(Default 14일)
    
    **참고**: 별도 정보가 없는 경우, Default 값으로 사용
    """

    return await ltcdrg_service.check_dementia_due_date(dementia_check_type, reference, check_months, prior_to)



from .ltcdrglist_tools import ltcdrglist_tools
from .advice_tools import advice_tools
from .patient_info_tools import patient_info_tools
from .ns_tools import ns_tools
from .progress_note_tools import progress_note_tools
from .vs_tools import vs_tools
from .jy_tools import jy_tools
from .util_tools import util_tools


from sqlalchemy import String
from cs_mcp.db.models.base import Base
from typing import Optional
from cs_mcp.db.sqlalchemy import mc, M


class Wj(Base):
  __tablename__ = "wj"  # type: ignore

  wj_auto: M[Optional[int]] = mc(
      default=None, primary_key=True, autoincrement=True
  )
  wj_chart: M[str] = mc(String(8))
  wj_suname: M[str] = mc(String(50))
  wj_birthday: M[str] = mc(String(8))



import uuid


class TempDict:
  def __init__(self):
    self._data = {}

  def create_key(self):
    return str(uuid.uuid4())

  def set(self, value):
    key = self.create_key()
    self._data[key] = value
    return key

  def get(self, key, default=None):
    return self._data.get(key, default)

  def delete(self, key):
    if key in self._data:
      del self._data[key]

  def clear(self):
    self._data.clear()


temp_dict = TempDict()



from typing import Any
from fastmcp import FastMCP
from sqlalchemy import select

from cs_mcp.db.dependencies import get_db
from cs_mcp.db.models import Ns
from cs_mcp.tools.types import SymdType,ChartNumberType, EymdType


def ns_tools(mcp: FastMCP[Any]):
  @mcp.tool()
  async def search_nursing_record(
      chart: ChartNumberType,
      symd: SymdType,
      eymd: EymdType,
  ):
    """간호기록을 조회합니다.
       간호기간은 종료일자로부터 최대 1달로 제한합니다.

    args:
        chart: 차트번호(8자리 숫자)
        start: 시작일자(yyyyMMdd)
        end: 종료일자(yyyyMMdd)
    returns:
        일자: 기록일자
        간호문제: 간호문제
        간호처치: 간호처치
    """

    async with get_db() as session:

      # session 에서 wj 객체를 가져온다.
      ns = await session.execute(
          select(Ns).where(
              (Ns.ns_chart == chart)
              & (Ns.ns_ymd >= symd)
              & (Ns.ns_ymd <= eymd)
          )
      )
      nss = ns.scalars()

    return [
        {"일자": ns.ns_ymd, "간호문제:": ns.ns_neyong1, "간호처치": ns.ns_neyong2}
        for ns in nss
    ]



from typing import Annotated, Any
from fastmcp import FastMCP
from typing import Annotated
from pydantic import Field
from sqlalchemy import select
from sqlalchemy.orm import load_only
from cs_mcp.db.dependencies import get_db
from cs_mcp.db.models import Wj
from cs_mcp.tools.types import ChartNumberType


def patient_info_tools(mcp: FastMCP[Any]):
  @mcp.tool()
  async def get_birth_by_chartnum(
      chart: Annotated[str, Field(description="차트번호임.")],
  ) -> str:
    """생년월일을 가져온다. 차트번호를 불러오기 위해서는 get_chart_by_name 를 호출하시오.
    args:
        chart (str): 차트번호
    """

    print("get_birth_by_chartnum called:", chart)
    async with get_db() as session:

      # session 에서 wj 객체를 가져온다.
      wj = await session.execute(
          select(Wj)
          .where(Wj.wj_chart == chart)
          .options(load_only(Wj.wj_birthday))
      )

      wj = wj.scalar()
      return wj.wj_birthday if wj is not None else ""

  @mcp.tool()
  async def get_charts_by_name(name: str):
    """이름 정보로 차트번호를 가져온다.
    args:
        name (str): 이름
    """

    print(f"get_charts_by_name called: {name}")

    async with get_db() as session:

      # session 에서 wj 객체를 가져온다.
      wj = await session.execute(
          select(Wj)
          .where(Wj.wj_suname == name)
          .options(load_only(Wj.wj_chart))
      )
    return [wj.wj_chart for wj in wj.scalars()]

  @mcp.tool()
  async def get_name_by_chart(charts: Annotated[list[ChartNumberType], Field(description="차트번호 목록")]):
    """차트번호로 이름을 가져온다.
    args:
        charts (list[str]): 차트번호 목록
    returns:
        Markdown table with 차트번호 and 이름
    """

    print(f"get_name_by_chart called: {charts}")

    results = []
    async with get_db() as session:
      for chart in charts:
        # session 에서 wj 객체를 가져온다.
        wj = await session.scalar(
            select(Wj)
            .where(Wj.wj_chart == chart)
            .options(load_only(Wj.wj_suname))
        )
        name = wj.wj_suname if wj is not None else ""
        results.append([chart, name])

    import pandas as pd
    df = pd.DataFrame(results, columns=["차트번호", "이름"])
    return df.to_markdown(index=False)



from contextlib import asynccontextmanager
import logging
from typing import AsyncGenerator
from sqlalchemy.ext.asyncio import AsyncSession, create_async_engine, async_sessionmaker

from cs_mcp.configs import config

# SQLAlchemy 로깅 설정 수정
logging.getLogger("sqlalchemy.engine").setLevel(logging.INFO)
# 중복 로깅 방지를 위해 propagate 설정 비활성화
logging.getLogger("sqlalchemy.engine").propagate = False

engine = create_async_engine(
    config.DATABASE_URL,
    echo=True,
    pool_pre_ping=True,  # 연결 사용 전 ping으로 유효성 검사
    # pool_recycle=60,
    # pool_timeout=30,
)

async_session = async_sessionmaker(
    engine,
    expire_on_commit=False,
    autoflush=False,
)


# 의존성 주입을 위한 비동기 세션 제공자 with 문으로 사용할 수 있게
@asynccontextmanager
async def get_db() -> AsyncGenerator[AsyncSession, None]:  
  # 데이터베이스 및 테이블 생성 (필요시)
  # async with engine.begin() as conn:
  #   meta = MetaData()
  #   await conn.run_sync(meta.create_all)
  async with async_session() as db:
    try:
      yield db
    finally:
      await db.close()


# @asynccontextmanager
# async def get_asession() -> AsyncGenerator[AsyncSession, None]:
#   async with async_session_maker() as session:
#     try:
#       print(f"💡[ 세션 생성 완료 ] {session}")
#       yield session
#       await session.commit()
#       print(f"💡[ 세션 커밋 완료 ] {session}")
#     except SQLAlchemyError as e:
#       print(f"🚨[ DB 오류, 롤백 ] {e}")
#       await session.rollback()
#       # 프로덕션에서는 에러 로깅 후 적절한 HTTPException 발생 고려

#     except Exception as e:
#       print(f"🚨[ 예외 발생, 롤백 ] {e}")
#       await session.rollback()
#       raise  # 원래 예외 다시 발생
#     finally:
#       await session.close()
#       print(f"💡[ 세션 종료 ] {session}")



from datetime import datetime, timedelta, timezone
from typing import Any
from fastmcp import FastMCP

import json
from typing import Annotated, Literal
from pydantic import Field
from cs_mcp.libs import temp_dict


def util_tools(mcp: FastMCP[Any]):
  @mcp.tool()
  async def get_today():
    """현재 일시를 반환합니다. 시간 정보가 필요한데 없으면 이 툴을 호출하세요.

    다음 키워드에 주로 사용:
    - 현재
    - 이번(주, 달)    
    - 오늘
    - 지금
    """
    # 한국 표준시 (KST, UTC+9)로 현재 시간 출력
    kst = timezone(timedelta(hours=9))
    return datetime.now(kst).strftime("%Y.%m.%d %H:%M:%S")

  @mcp.tool()
  async def clear_temp():
    """TempKey를 사용 시 메모리를 해제합니다. 반드시 마지막에 한번만 호출하세요."""
    temp_dict.clear()
    return {"status": "cleared"}

  @mcp.tool()
  async def compare_two_files(
      temp_key_a:  Annotated[str, Field(description="첫 번째 TempKey")],
      temp_key_b: Annotated[str, Field(description="두 번째 TempKey")],
      # 차집합,교집합 매개변수
      kind: Annotated[
          Literal["intersection", "only_in_a", "only_in_b"], Field(
          description="분석할 집합의 종류 (intersection: 두 집합 모두에 있는 값, only_in_a: 첫 번째 집합에만 있는 값, only_in_b: 두 번째 집합에만 있는 값)")]
  ):
    """두 TempKey에 해당하는 데이터를 비교하여 intersection or 차집합을 분석합니다."""
    a = temp_dict.get(temp_key_a)
    b = temp_dict.get(temp_key_b)
    if a is None or b is None:
      return {"error": "Invalid TempKey"}
    a_set = set(a)
    b_set = set(b)

    result = []
    if kind == "intersection":
      intersection = list(a_set & b_set)
      result = {"intersection": intersection}
    elif kind == "only_in_a":
      only_in_a = list(a_set - b_set)
      result = {"only_in_a": only_in_a}
    elif kind == "only_in_b":
      only_in_b = list(b_set - a_set)
      result = {"only_in_b": only_in_b}

    return f"""결과:
{json.dumps(result, ensure_ascii=False, indent=2)}

---
모든 작업이 완료되면 반드시 "clear_temp" 툴을 호출하여 메모리를 해제하세요.
"""



from typing import Optional
from pydantic import BaseModel, Field


class BatchVitalSignData(BaseModel):
  hulap2: Optional[str] = Field(
      default=None, description="수축기 혈압(hulap2)")
  hulap1: Optional[str] = Field(
      default=None, description="이완기 혈압(hulap1)")
  maekbak: Optional[str] = Field(default=None, description="맥박(maekbak)")
  hohup: Optional[str] = Field(default=None, description="호흡수(hohup)")
  cheon: Optional[str] = Field(default=None, description="체온(cheon)")
  weight: Optional[str] = Field(default=None, description="체중(weight)")
  height: Optional[str] = Field(default=None, description="신장(height)")
  spo2: Optional[str] = Field(default=None, description="산소포화도(spo2)")
  # intake 필드의 description이 "맥박 산소포화도"로 되어 있는데, 보통 "섭취량"을 의미합니다. 확인 필요.
  # spo2와 중복되는 것 같습니다. 만약 섭취량이라면 description을 수정하세요.
  intake: Optional[str] = Field(
      default=None, description="섭취량(intake) 또는 다른 의미라면 수정"
  )
  urine: Optional[str] = Field(default=None, description="소변량(urine)")
  blood: Optional[str] = Field(default=None, description="혈액량(blood)")
  aspiration: Optional[str] = Field(
      default=None, description="흡인량(aspiration)"
  )
  drainage: Optional[str] = Field(
      default=None, description="배액량(drainage)")
  vomitus: Optional[str] = Field(
      default=None, description="구토량(vomitus)")



# from typing import Any, Literal, Optional
# from pydantic import BaseModel
# import win32file


# class PipeSendData(BaseModel):
#     """Data to be sent to the pipe."""

#     type: Literal["open_form", "load_chart", "ping", "write_progress_note", "write_batch_vital_sign"]
#     target: Optional[str] = None
#     message: Optional[str] = None
#     data: Optional[dict[str, Any]] = None


# class PipeClient:
#     def __init__(self, pipe_name="메인화면_pipe"):
#         self.pipe_name = rf"\\.\pipe\{pipe_name}"
#         self.pipe = None

#     def __enter__(self):
#         self.pipe = win32file.CreateFile(
#             self.pipe_name,
#             win32file.GENERIC_READ | win32file.GENERIC_WRITE,
#             0,
#             None,
#             win32file.OPEN_EXISTING,
#             0,
#             None,
#         )
#         return self  # with 문의 as 구문에 전달될 객체

#     def __exit__(self, exc_type, exc_val, exc_tb):
#         if self.pipe:
#             self.pipe.close()  # type: ignore
#             self.pipe = None  # 파이프 닫았음을 명시

#     def read_message(self):
#         data: bytes
#         result, data = win32file.ReadFile(self.pipe, 4096)  # type: ignore
#         return data.decode("utf-8")

#     def send(self, data: PipeSendData):
#         win32file.WriteFile(self.pipe, data.model_dump_json().encode("utf-8"))  # type: ignore


# # with PipeServer("test_pipe") as pipe:
# #     # 응답 전송
# #     response = "서버에서 보낸 응답"
# #     pipe.send_message(response)
# #     print("응답 전송 완료")

# #     # 메시지 수신
# #     message = pipe.read_message()
# #     print(f"수신된 메시지: {message}")
